
-- Components description
-- id - unique id of the memory record in cheat table
-- modifier - value to be added to the value of the memory record
-- valFromFunc - this func will be called to fill the value of the component
-- components_inheriting_value - these components will have the same value as the parent
-- depends_on - components used to calc new value for the parent
-- group - player attribute group
-- events - component events
function get_components_description_player_edit()
    return {
        PlayerIDEdit = {id = CT_MEMORY_RECORDS['PLAYERID'], modifier = 0},
        TeamIDEdit = {id = CT_MEMORY_RECORDS['TPLINKS_TEAMID'], modifier = 1},
        OverallEdit = {id = 119, modifier = 1},
        PotentialEdit = {id = 37, modifier = 1},
        
        AgeEdit = {
            id = CT_MEMORY_RECORDS['BIRTHDATE'],
            modifier = 0,
            valFromFunc = birthdate_to_age,
            onApplyChanges = age_to_birthdate
        },
        FirstNameIDEdit = {id = 51, modifier = 0},
        LastNameIDEdit = {id = 74, modifier = 0},
        CommonNameIDEdit = {id = 129, modifier = 0},
        JerseyNameIDEdit = {id = 126, modifier = 0},
        NationalityCB = {id = 101, modifier = 0},
        ContractValidUntilEdit = {id = 43, modifier = 0},
        PlayerJoinTeamDateEdit = {
            id = 97,
            modifier = 0,
            valFromFunc = days_to_date,
            onApplyChanges = date_to_days
        },
        JerseyNumberEdit = {id = 3524, modifier = 1},
        GKSaveTypeEdit = {id = 21, modifier = 0},
        GKKickStyleEdit = {id = 108, modifier = 0},

        PreferredPosition1CB = {id = 60, modifier = 0},
        PreferredPosition2CB = {id = 56, modifier = -1},
        PreferredPosition3CB = {id = 26, modifier = -1},
        PreferredPosition4CB = {id = 133, modifier = -1},
    
        AttackingWorkRateCB = {id = 83, modifier = 0},
        DefensiveWorkRateCB = {id = 99, modifier = 0},
        SkillMovesCB = {id = 79, modifier = 0},
        WeakFootCB = {id = 104, modifier = 1},
    
        IsRetiringCB = {id = 31, modifier = 0},
        InternationalReputationCB = {id = 75, modifier = 1},
        PreferredFootCB = {id = 102, modifier = 1},
        GenderCB = {id = 71, modifier = 0},
        
        AttackTrackBar = {
            valFromFunc = AttributesTrackBarVal,
            group = 'Attack',
            components_inheriting_value = {
                "AttackValueLabel",
            },
            depends_on = {
                "CrossingEdit", "FinishingEdit", "HeadingAccuracyEdit",
                "ShortPassingEdit", "VolleysEdit"
            },
            events = {
                OnChange = AttributesTrackBarOnChange,
            },
        },
        CrossingEdit = {
            id = 36, 
            modifier = 1,
            group = 'Attack',
            events = {
                OnChange = AttrOnChange,
            },
        },
        FinishingEdit = {
            id = 45, 
            modifier = 1,
            group = 'Attack',
            events = {
                OnChange = AttrOnChange,
            },
        },
        HeadingAccuracyEdit = {
            id = 87, 
            modifier = 1,
            group = 'Attack',
            events = {
                OnChange = AttrOnChange,
            },
        },
        ShortPassingEdit = {
            id = 77,
            modifier = 1,
            group = 'Attack',
            events = {
                OnChange = AttrOnChange,
            },
        },
        VolleysEdit = {
            id = 134,
            modifier = 1,
            group = 'Attack',
            events = {
                OnChange = AttrOnChange,
            },
        },
        
        DefendingTrackBar = {
            valFromFunc = AttributesTrackBarVal,
            group = 'Defending',
            components_inheriting_value = {
                "DefendingValueLabel",
            },
            depends_on = {
                "MarkingEdit", "StandingTackleEdit", "SlidingTackleEdit",
            },
            events = {
                OnChange = AttributesTrackBarOnChange,
            },
        },
        MarkingEdit = {
            id = 111, 
            modifier = 1,
            group = 'Defending',
            events = {
                OnChange = AttrOnChange,
            },
        },
        StandingTackleEdit = {
            id = 25, 
            modifier = 1,
            group = 'Defending',
            events = {
                OnChange = AttrOnChange,
            },
        },
        SlidingTackleEdit = {
            id = 47,
            modifier = 1,
            group = 'Defending',
            events = {
                OnChange = AttrOnChange,
            },
        },
        
        SkillTrackBar = {
            valFromFunc = AttributesTrackBarVal,
            group = 'Skill',
            components_inheriting_value = {
                "SkillValueLabel",
            },
            depends_on = {
                "DribblingEdit", "CurveEdit", "FreeKickAccuracyEdit",
                "LongPassingEdit", "BallControlEdit",
            },
            events = {
                OnChange = AttributesTrackBarOnChange,
            },
        },
        DribblingEdit = {
            id = 46, 
            modifier = 1,
            group = 'Skill',
            events = {
                OnChange = AttrOnChange,
            },
        },
        CurveEdit = {
            id = 16, 
            modifier = 1,
            group = 'Skill',
            events = {
                OnChange = AttrOnChange,
            },
        },
        FreeKickAccuracyEdit = {
            id = 78, 
            modifier = 1,
            group = 'Skill',
            events = {
                OnChange = AttrOnChange,
            },
        },
        LongPassingEdit = {
            id = 27,
            modifier = 1,
            group = 'Skill',
            events = {
                OnChange = AttrOnChange,
            },
        },
        BallControlEdit = {
            id = 62,
            modifier = 1,
            group = 'Skill',
            events = {
                OnChange = AttrOnChange,
            },
        },
        
        GoalkeeperTrackBar = {
            valFromFunc = AttributesTrackBarVal,
            group = 'Goalkeeper',
            components_inheriting_value = {
                "GoalkeeperValueLabel",
            },
            depends_on = {
                "GKDivingEdit", "GKHandlingEdit", "GKKickingEdit",
                "GKPositioningEdit", "GKReflexEdit",
            },
            events = {
                OnChange = AttributesTrackBarOnChange,
            },
        },
        GKDivingEdit = {
            id = 33, 
            modifier = 1,
            group = 'Goalkeeper',
            events = {
                OnChange = AttrOnChange,
            },
        },
        GKHandlingEdit = {
            id = 92,
            modifier = 1,
            group = 'Goalkeeper',
            events = {
                OnChange = AttrOnChange,
            },
        },
        GKKickingEdit = {
            id = 73,
            modifier = 1,
            group = 'Goalkeeper',
            events = {
                OnChange = AttrOnChange,
            },
        },
        GKPositioningEdit = {
            id = 113,
            modifier = 1,
            group = 'Goalkeeper',
            events = {
                OnChange = AttrOnChange,
            },
        },
        GKReflexEdit = {
            id = 38,
            modifier = 1,
            group = 'Goalkeeper',
            events = {
                OnChange = AttrOnChange,
            },
        },
        
        PowerTrackBar = {
            valFromFunc = AttributesTrackBarVal,
            group = 'Power',
            components_inheriting_value = {
                "PowerValueLabel",
            },
            depends_on = {
                "ShotPowerEdit", "JumpingEdit", "StaminaEdit",
                "StrengthEdit", "LongShotsEdit",
            },
            events = {
                OnChange = AttributesTrackBarOnChange,
            },
        },
        ShotPowerEdit = {
            id = 63,
            modifier = 1,
            group = 'Power',
            events = {
                OnChange = AttrOnChange,
            },
        },
        JumpingEdit = {
            id = 105,
            modifier = 1,
            group = 'Power',
            events = {
                OnChange = AttrOnChange,
            },
        },
        StaminaEdit = {
            id = 109,
            modifier = 1,
            group = 'Power',
            events = {
                OnChange = AttrOnChange,
            },
        },
        StrengthEdit = {
            id = 57,
            modifier = 1,
            group = 'Power',
            events = {
                OnChange = AttrOnChange,
            },
        },
        LongShotsEdit = {
            id = 32,
            modifier = 1,
            group = 'Power',
            events = {
                OnChange = AttrOnChange,
            },
        },
        
        MovementTrackBar = {
            valFromFunc = AttributesTrackBarVal,
            group = 'Movement',
            components_inheriting_value = {
                "MovementValueLabel",
            },
            depends_on = {
                "AccelerationEdit", "SprintSpeedEdit", "AgilityEdit",
                "ReactionsEdit", "BalanceEdit",
            },
            events = {
                OnChange = AttributesTrackBarOnChange,
            },
        },
        AccelerationEdit = {
            id = 86,
            modifier = 1,
            group = 'Movement',
            events = {
                OnChange = AttrOnChange,
            },
        },
        SprintSpeedEdit = {
            id = 52, 
            modifier = 1,
            group = 'Movement',
            events = {
                OnChange = AttrOnChange,
            },
        },
        AgilityEdit = {
            id = 18,
            modifier = 1,
            group = 'Movement',
            events = {
                OnChange = AttrOnChange,
            },
        },
        ReactionsEdit = {
            id = 40,
            modifier = 1,
            group = 'Movement',
            events = {
                OnChange = AttrOnChange,
            },
        },
        BalanceEdit = {
            id = 70,
            modifier = 1,
            group = 'Movement',
            events = {
                OnChange = AttrOnChange,
            },
        },
        
        MentalityTrackBar = {
            valFromFunc = AttributesTrackBarVal,
            group = 'Mentality',
            components_inheriting_value = {
                "MentalityValueLabel",
            },
            depends_on = {
                "AggressionEdit", "ComposureEdit", "InterceptionsEdit",
                "AttackPositioningEdit", "VisionEdit", "PenaltiesEdit",
            },
            events = {
                OnChange = AttributesTrackBarOnChange,
            },
        },
        AggressionEdit = {
            id = 85,
            modifier = 1,
            group = 'Mentality',
            events = {
                OnChange = AttrOnChange,
            },
        },
        ComposureEdit = {
            id = 41,
            modifier = 1,
            group = 'Mentality',
            events = {
                OnChange = AttrOnChange,
            },
        },
        InterceptionsEdit = {
            id = 34,
            modifier = 1,
            group = 'Mentality',
            events = {
                OnChange = AttrOnChange,
            },
        },
        AttackPositioningEdit = {
            id = 22,
            modifier = 1,
            group = 'Mentality',
            events = {
                OnChange = AttrOnChange,
            },
        },
        VisionEdit = {
            id = 42,
            modifier = 1,
            group = 'Mentality',
            events = {
                OnChange = AttrOnChange,
            },
        },
        PenaltiesEdit = {
            id = 28,
            modifier = 1,
            group = 'Mentality',
            events = {
                OnChange = AttrOnChange,
            },
        },
    
        LongThrowInCB = {id = 64,},
        PowerFreeKickCB = {id = 178,},
        InjuryProneCB = {id = 179,},
        SolidPlayerCB = {id = 180,},
        LeadershipCB = {id = 183,},
        EarlyCrosserCB = {id = 184,},
        FinesseShotCB = {id = 185,},
        FlairCB = {id = 186,},
        SpeedDribblerCB = {id = 169,},
        GKLongthrowCB = {id = 171,},
        PowerheaderCB = {id = 172,},
        GiantthrowinCB = {id = 173,},
        OutsitefootshotCB = {id = 174,},
        SwervePassCB = {id = 165,},
        SecondWindCB = {id = 175,},
        FlairPassesCB = {id = 176,},

        BicycleKicksCB = {id = 177,},
        GKFlatKickCB = {id = 162,},
        OneClubPlayerCB = {id = 163,},
        TeamPlayerCB = {id = 164,},
        RushesOutOfGoalCB = {id = 161,},
        CautiousWithCrossesCB = {id = 160,},
        ComesForCrossessCB = {id = 159,},
        SaveswithFeetCB = {id = 208,},
        SetPlaySpecialistCB = {id = 209,},
        DivesIntoTacklesCB = {id = 181,},
        LongPasserCB = {id = 187,},
        LongShotTakerCB = {id = 168,},
        PlaymakerCB = {id = 170,},
        ChipShotCB = {id = 166,},
        TechnicalDribblerCB = {id = 167,},
    
        HeightEdit = {id = 53, modifier = 130},
        WeightEdit = {id = 66, modifier = 30},
        BodyTypeCB = {id = 130, modifier = 1},

        TattooHeadEdit = {id = 55, modifier = 0},
        TattooFrontEdit = {id = 100, modifier = 0},
        TattooBackEdit = {id = 19, modifier = 0},
        TattooRightArmEdit = {id = 69, modifier = 0},
        TattooLeftArmEdit = {id = 23, modifier = 0},
        TattooRightLegEdit = {id = 136, modifier = 0},
        TattooLeftLegEdit = {id = 61, modifier = 0},

        HeadTypeCodeCB = {
            id = CT_MEMORY_RECORDS['HEADTYPECODE'],
            modifier = 0,
            already_filled = true,
            events = {
                OnChange = HeadTypeCodeCBOnChange,
                OnDropDown = CommonCBOnDropDown,
                OnMouseEnter = CommonCBOnMouseEnter,
                OnMouseLeave = CommonCBOnMouseLeave,
            },
        },
        HeadTypeGroupCB = {
            valFromFunc = FillHeadTypeCB,
            events = {
                OnChange = HeadTypeGroupCBOnChange,
                OnDropDown = CommonCBOnDropDown,
                OnMouseEnter = CommonCBOnMouseEnter,
                OnMouseLeave = CommonCBOnMouseLeave,
            },
        },
        HairTypeEdit = {id = 24, modifier = 0},
        HairStyleEdit = {id = 128, modifier = 0},
        HairColorCB = {
            id = CT_MEMORY_RECORDS['HAIRCOLORCODE'], 
            modifier = 0,
            events = {
                OnChange = HeadTypeCodeCBOnChange,
                OnDropDown = CommonCBOnDropDown,
                OnMouseEnter = CommonCBOnMouseEnter,
                OnMouseLeave = CommonCBOnMouseLeave,
            },
        },
        FacialHairTypeEdit = {id = 15, modifier = 0},
        FacialHairColorEdit = {id = 137, modifier = 0},
        SideburnsEdit = {id = 103, modifier = 0},
        EyebrowEdit = {id = 89, modifier = 0},
        EyeColorEdit = {id = 93, modifier = 0},
        SkinTypeEdit = {id = 89, modifier = 0},
        SkinColorCB = {
            id = CT_MEMORY_RECORDS['SKINTONECODE'],
            modifier = 1,
            events = {
                OnChange = HeadTypeCodeCBOnChange,
                OnDropDown = CommonCBOnDropDown,
                OnMouseEnter = CommonCBOnMouseEnter,
                OnMouseLeave = CommonCBOnMouseLeave,
            },
        },

        HasHighQualityHeadCB = {id = 67, modifier = 0},
        HeadAssetIDEdit = {id = 72, modifier = 0},
        HeadVariationEdit = {id = 114, modifier = 0},
        HeadClassCodeEdit = {id = 98, modifier = 0},

        JerseyStyleEdit = {id = 17, modifier = 0},
        JerseyFitEdit = {id = 123, modifier = 0},
        jerseysleevelengthEdit = {id = 94, modifier = 0},
        hasseasonaljerseyEdit = {id = 54, modifier = 0},
        shortstyleEdit = {id = 118, modifier = 0},
        socklengthEdit = {id = 65, modifier = 0},
    
        GKGloveTypeEdit = {id = 68, modifier = 0},
        shoetypeEdit = {id = 58, modifier = 0},
        shoedesignEdit = {id = 125, modifier = 0},
        shoecolorEdit1 = {id = 127, modifier = 0},
        shoecolorEdit2 = {id = 35, modifier = 0},
        AccessoryEdit1 = {id = 96, modifier = 0},
        AccessoryColourEdit1 = {id = 49, modifier = 0},
        AccessoryEdit2 = {id = 124, modifier = 0},
        AccessoryColourEdit2 = {id = 135, modifier = 0},
        AccessoryEdit3 = {id = 48, modifier = 0},
        AccessoryColourEdit3 = {id = 95, modifier = 0},
        AccessoryEdit4 = {id = 20, modifier = 0},
        AccessoryColourEdit4 = {id = 112, modifier = 0},
    
        runningcodeEdit1 = {id = 132, modifier = 0},
        runningcodeEdit2 = {id = 90, modifier = 0},
        FinishingCodeEdit1 = {id = 39, modifier = 0},
        FinishingCodeEdit2 = {id = 84, modifier = 0},
        AnimFreeKickStartPosEdit = {id = 29, modifier = 0},
        AnimPenaltiesStartPosEdit = {id = 131, modifier = 0},
        AnimPenaltiesKickStyleEdit = {id = 30, modifier = 0},
        AnimPenaltiesMotionStyleEdit = {id = 76, modifier = 0},
        AnimPenaltiesApproachEdit = {id = 44, modifier = 0},
        FacePoserPresetEdit = {id = 80, modifier = 0},
        EmotionEdit = {id = 121, modifier = 0},
        SkillMoveslikelihoodEdit = {id = 115, modifier = 0},
        ModifierEdit = {id = 91, modifier = -5},
        IsCustomizedEdit = {id = 88, modifier = 0},
        UserCanEditNameEdit = {id = 81, modifier = 0},
        RunStyleEdit = {id = 122, modifier = 0},

        -- career_playercontract
        WageEdit = {
            id=CT_MEMORY_RECORDS['CONTRACT_WAGE'], 
            modifier = 0,
            custom_apply_changes = true
        },
        SquadRoleCB = {
            id=CT_MEMORY_RECORDS['CONTRACT_PLAYER_ROLE'],
            modifier = -1,
            already_filled = true,
            custom_apply_changes = true,
            events = {
                OnChange = CommonCBOnChange,
                OnDropDown = CommonCBOnDropDown,
                OnMouseEnter = CommonCBOnMouseEnter,
                OnMouseLeave = CommonCBOnMouseLeave,
            }
        }, -- Stored in us002
        LoanWageSplitEdit = {
            id=CT_MEMORY_RECORDS['CONTRACT_LOAN_WAGE_SPLIT'],
            custom_apply_changes = true,
            modifier = -1
        },
        PerformanceBonusTypeCB = {
            id=CT_MEMORY_RECORDS['CONTRACT_PERFORMANCE_BONUS_TYPE'], 
            modifier = 0, 
            already_filled = true,
            custom_apply_changes = true,
            events = {
                OnChange = PerformanceBonusTypeCBChange,
                OnDropDown = CommonCBOnDropDown,
                OnMouseEnter = CommonCBOnMouseEnter,
                OnMouseLeave = CommonCBOnMouseLeave,
            }
        },
        PerformanceBonusCountEdit = {
            id=CT_MEMORY_RECORDS['CONTRACT_PERFORMANCE_BONUS_COUNT'],
            modifier = -1,
            custom_apply_changes = true,
            valFromFunc=FillPerformanceBonusCountEdit
        },
        PerformanceBonusValueEdit = {
            id=CT_MEMORY_RECORDS['CONTRACT_PERFORMANCE_BONUS_VALUE'],
            custom_apply_changes = true,
            modifier = -1
        },
    }
end

HEAD_TYPE_GROUPS = {
    African_South_European = {
        5000,
        5001,
        5002,
        5003,
        10500,
        10501,
        10502,
    },
    African = {
        1000,
        1001,
        1002,
        1003,
        1004,
        1005,
        1006,
        1007,
        1008,
        1009,
        1010,
        1011,
        1012,
        1013,
        1014,
        1015,
        1016,
        1017,
        1018,
        1019,
        1020,
        1021,
        1022,
        1023,
        1024,
        1025,
        1026,
        1027,
        6500,
        6501,
        6502,
    },
    Eastern_European = {
        2000,
        2001,
        2002,
        2003,
        2004,
        2005,
        2006,
        2007,
        2008,
        2009,
        2010,
        2011,
        2012,
        2013,
        2014,
        2015,
        2016,
        2017,
        2019,
        2020,
        2021,
        2022,
        2023,
        2024,
        2025,
        2026,
        2027,
        2028,
        2029,
        2030,
        7500,
        7501,
        7502,
    },
    South_European = {
        3500,
        3501,
        3502,
        3503,
        3504,
        3505,
        9000,
        9001,
        9002,
    },
    Arabic = {
        2500,
        2501,
        2502,
        2503,
        2504,
        2505,
        2506,
        2507,
        2508,
        2509,
        2510,
        2511,
        2512,
        2513,
        2514,
        2515,
        2516,
        2517,
        2518,
        8000,
        8001,
        8002,
    },
    Asian = {
        500,
        501,
        502,
        503,
        504,
        505,
        506,
        507,
        508,
        509,
        510,
        511,
        512,
        513,
        514,
        515,
        516,
        517,
        518,
        519,
        520,
        521,
        522,
        523,
        524,
        525,
        526,
        527,
        528,
        529,
        530,
        531,
        532,
        533,
        534,
        535,
        536,
        537,
        538,
        539,
        540,
        541,
        542,
        543,
        544,
        545,
        546,
        547,
        548,
        549,
        550,
        551,
        552,
        553,
        554,
        555,
        556,
        557,
        558,
        559,
        560,
        561,
        562,
        6000,
        6001,
        6002,
        6003,
        6004,
        6005,
        6006,
        6007,
        6008,
        6009,
    },
    Caucasian = {
        0,
        1,
        2,
        3,
        4,
        5,
        6,
        7,
        8,
        9,
        10,
        11,
        12,
        13,
        14,
        15,
        16,
        17,
        18,
        19,
        20,
        21,
        22,
        23,
        24,
        25,
        5500,
        5501,
        5502,
        5503,
        5504,
        5505,
    },
    Latin = {
        1500,
        1501,
        1502,
        1503,
        1504,
        1505,
        1506,
        1507,
        1508,
        1509,
        1510,
        1511,
        1512,
        1513,
        1514,
        1515,
        1516,
        1517,
        1518,
        1519,
        1520,
        1521,
        1522,
        1523,
        1524,
        1525,
        1526,
        1527,
        1528,
        7000,
        7001,
        7002,
        7003,
        7004,
        7005,
        7006,
        7007,
        7008,
        7009,
        7010,
        7011,
    },
    African_South = {
        3000,
        3001,
        3002,
        3003,
        3004,
        3005,
        8500,
        8501,
        8502,
    },
    North_European = {
        4000,
        4001,
        4002,
        4003,
        9500,
        9501,
        9502,
    },
    African_North = {
        4500,
        4501,
        4502,
        4525,
        10000,
        10001,
        10002,
    }
}

PLAYERS_TAB_PANEL_MAP = {
    PlayerInfoTab = "PlayerInfoPanel",
    AttributesTab = "AttributesPanel",
    TraitsTab = "TraitsPanel",
    AppearanceTab = "AppearancePanel",
    AccessoriesTab = "AccessoriesPanel",
    OtherTab = "OtherPanel",
    PlayerCloneTab = "PlayerClonePanel"
}
